==> comedi-0.7.28.tgz: comedi-0.7.28/module/ni-dio.c <==
/*
    module/ni-dio.c
    driver for National Instruments PCI-DIO-96
               National Instruments PCI-DIO-32HS

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1999 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
   This driver is for both the NI PCI-DIO-32HS and the PCI-DIO-96,
   which have very different architectures.  But, since the '96 is
   so simple, it is included here.
 */

#include <linux/kernel.h>
#include <comedi_module.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/malloc.h>
#include <linux/delay.h>
#include <linux/pci.h>
#include <8255.h>

/* general debugging messages */
#define DEBUG


#define PCI_VENDOR_ID_NATINST	0x1093

#define PCI_DIO_SIZE 4096
#define PCI_MITE_SIZE 4096

/* defines for the PCI-DIO-96 */

#define NIDIO_A 0
#define NIDIO_B 4
#define NIDIO_C 8
#define NIDIO_D 12

/* defines for the PCI-DIO-32HS */

#define Chip_ID_D			24
#define Chip_ID_I			25
#define Chip_ID_O			26
#define Chip_Version			27

#define Port_IO(x)			(28+(x))
#define Port_Pin_Directions(x)		(32+(x))
#define Port_Pin_Mask(x)		(36+(x))
#define Port_Pin_Polarities(x)		(40+(x))
#define Port_Pattern(x)			(48+(x))


static int nidio_rem(comedi_device *dev);

typedef struct{
	int dev_id;
	char *name;
	int n_8255;
	int is_diodaq;
}nidio_board;
static nidio_board nidio_boards[]={
	{
	dev_id:		0x1150,
	name:		"pci-dio-32hs",
	n_8255:		0,
	is_diodaq:	1,
	},
	{
	dev_id:		0x0160,
	name:		"pci-dio-96",
	n_8255:		4,
	is_diodaq:	0,
	},
};
#define n_nidio_boards 2

typedef struct{
	struct pci_dev *pci_device;
	unsigned long daq_phys_addr;
	unsigned long mite_phys_addr;
	void *daq_io_addr;
	void *mite_io_addr;
	int boardtype;
	int dio;
	int aip[16];
}nidio96_private;
#define devpriv ((nidio96_private *)dev->private)

static int nidio_find_device(comedi_device *dev,comedi_devconfig *it);
static int setup_mite(comedi_device *dev);


static int nidio96_8255_cb(int dir,int port,int data,void *arg)
{
	int iobase=(int)arg;

	if(dir){
		writeb(data,iobase+port);
		return 0;
	}else{
		return readb(iobase+port);
	}
}

static int nidio_dio(comedi_device *dev,comedi_subdevice *s,comedi_trig *it)
{
	if(it->flags & TRIG_CONFIG){
		do_pack(&s->io_bits,it);
		writel(s->io_bits,dev->iobase+Port_Pin_Directions(0));
	}else{
		if(it->flags&TRIG_WRITE){
			do_pack(&s->state,it);
			writel(s->state,dev->iobase+Port_IO(0));
		}else{
			unsigned int data;

			data=readl(dev->iobase+Port_IO(0));
			di_unpack(data,it);
		}
	}

	return it->n_chan;
}


int nidio_init(comedi_device *dev,comedi_devconfig *it)
{
	comedi_subdevice *s;
	int ret;
	
	if(strcmp("nidio",it->board_name))
		return 0;
	
	printk("comedi%d: nidio:",dev->minor);

	if((ret=alloc_private(dev,sizeof(nidio96_private)))<0)
		return ret;
	
	ret=nidio_find_device(dev,it);
	if(ret<0)goto out;

	ret=setup_mite(dev);
	if(ret<0)goto out;

	dev->driver_name="nidio";
	dev->board_name=nidio_boards[dev->board].name;
	dev->irq=0;
	printk(" %s",dev->board_name);

	dev->n_subdevices=1;
	if((ret=alloc_subdevices(dev))<0)
		return ret;

	if(!nidio_boards[dev->board].is_diodaq){
		subdev_8255_init(dev,dev->subdevices+0,nidio96_8255_cb,(void *)(dev->iobase+NIDIO_A));
		subdev_8255_init(dev,dev->subdevices+1,nidio96_8255_cb,(void *)(dev->iobase+NIDIO_B));
		subdev_8255_init(dev,dev->subdevices+2,nidio96_8255_cb,(void *)(dev->iobase+NIDIO_C));
		subdev_8255_init(dev,dev->subdevices+3,nidio96_8255_cb,(void *)(dev->iobase+NIDIO_D));
	}else{

		printk(" rev=%d",readb(dev->iobase+Chip_Version));
	
		s=dev->subdevices+0;

		s->type=COMEDI_SUBD_DIO;
		s->subdev_flags=SDF_READABLE|SDF_WRITEABLE|SDF_RT;
		s->n_chan=32;
		s->range_type=RANGE_digital;
		s->maxdata=1;
		s->trig[0]=nidio_dio;

		writel(0,dev->iobase+Port_IO(0));
		writel(0,dev->iobase+Port_Pin_Directions(0));
		writel(0,dev->iobase+Port_Pin_Mask(0));
	}

	dev->rem=nidio_rem;
	
	printk("\n");

	return 1;
out:
	nidio_rem(dev);

	return ret;
}

static int nidio_rem(comedi_device *dev)
{
	if(devpriv->mite_io_addr){
		iounmap(devpriv->mite_io_addr);
		devpriv->mite_io_addr=NULL;
	}
	if(devpriv->daq_io_addr){
		iounmap(devpriv->daq_io_addr);
		devpriv->daq_io_addr=NULL;
	}

	return 0;
}


static int nidio_find_device(comedi_device *dev,comedi_devconfig *it)
{
	struct pci_dev *pci_device=NULL;
	int i;
	
	if(!pci_present()){
		printk(" no PCI bus\n");
		return -EINVAL;
	}

	for(pci_device=pci_devices;pci_device;pci_device=pci_device->next){
		if(pci_device->vendor==PCI_VENDOR_ID_NATINST){
			for(i=0;i<n_nidio_boards;i++){
				if(pci_device->device==nidio_boards[i].dev_id)
					goto found;
			}
			printk("unknown NI device found with device_id=0x%04x\n",
				pci_device->device);
		}
	}
	printk("no device found\n");
	return -EINVAL;

found:
	dev->board=i;
	devpriv->pci_device=pci_device;

	return 0;
}


/* should merge this with PCI-MIO mite setup. */

static int setup_mite(comedi_device *dev)
{
	unsigned long			offset;
	u32				addr;

	addr=devpriv->pci_device->base_address[1];
	devpriv->daq_phys_addr=addr;
	offset = devpriv->daq_phys_addr & ~PAGE_MASK;
	devpriv->daq_io_addr = ioremap(devpriv->daq_phys_addr & PAGE_MASK, PCI_DIO_SIZE + offset )
		+ offset;
#ifdef PCI_DEBUG
	printk("DAQ:0x%08lx mapped to %p, ",devpriv->daq_phys_addr,devpriv->daq_io_addr);
#endif

	addr=devpriv->pci_device->base_address[0];
	devpriv->mite_phys_addr=addr;
	offset = devpriv->mite_phys_addr & ~PAGE_MASK;
	devpriv->mite_io_addr = ioremap(devpriv->mite_phys_addr & PAGE_MASK, PCI_MITE_SIZE + offset )
		+ offset;
#ifdef PCI_DEBUG
	printk("MITE:0x%08lx mapped to %p ",devpriv->mite_phys_addr,devpriv->mite_io_addr);
#endif
	
	/* XXX don't know what the 0xc0 and 0x80 mean */
	writel(devpriv->daq_phys_addr | 0x80 , devpriv->mite_io_addr + 0xc0 );
	
	dev->iobase = (int) devpriv->daq_io_addr;
	
	return 0;
}

==> comedi-0.7.30.tgz: comedi-0.7.30/module/ni-dio.c <==
/*
    module/ni-dio.c
    driver for National Instruments PCI-DIO-96
               National Instruments PCI-DIO-32HS

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1999 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
   This driver is for both the NI PCI-DIO-32HS and the PCI-DIO-96,
   which have very different architectures.  But, since the '96 is
   so simple, it is included here.
 */

#include <linux/kernel.h>
#include <comedi_module.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/malloc.h>
#include <linux/delay.h>
#include <mite.h>
#include <8255.h>

/* general debugging messages */
#define DEBUG


#define PCI_VENDOR_ID_NATINST	0x1093

#define PCI_DIO_SIZE 4096
#define PCI_MITE_SIZE 4096

/* defines for the PCI-DIO-96 */

#define NIDIO_A 0
#define NIDIO_B 4
#define NIDIO_C 8
#define NIDIO_D 12

/* defines for the PCI-DIO-32HS */

#define Chip_ID_D			24
#define Chip_ID_I			25
#define Chip_ID_O			26
#define Chip_Version			27

#define Port_IO(x)			(28+(x))
#define Port_Pin_Directions(x)		(32+(x))
#define Port_Pin_Mask(x)		(36+(x))
#define Port_Pin_Polarities(x)		(40+(x))
#define Port_Pattern(x)			(48+(x))


static int nidio_rem(comedi_device *dev);

typedef struct{
	int dev_id;
	char *name;
	int n_8255;
	int is_diodaq;
}nidio_board;
static nidio_board nidio_boards[]={
	{
	dev_id:		0x1150,
	name:		"pci-dio-32hs",
	n_8255:		0,
	is_diodaq:	1,
	},
	{
	dev_id:		0x0160,
	name:		"pci-dio-96",
	n_8255:		4,
	is_diodaq:	0,
	},
};
#define n_nidio_boards 2

typedef struct{
	struct mite_struct *mite;
	int boardtype;
	int dio;
	int aip[16];
}nidio96_private;
#define devpriv ((nidio96_private *)dev->private)

static int nidio_find_device(comedi_device *dev,comedi_devconfig *it);
#if 0
static int setup_mite(comedi_device *dev);
#endif


static int nidio96_8255_cb(int dir,int port,int data,void *arg)
{
	int iobase=(int)arg;

	if(dir){
		writeb(data,iobase+port);
		return 0;
	}else{
		return readb(iobase+port);
	}
}

static int nidio_dio(comedi_device *dev,comedi_subdevice *s,comedi_trig *it)
{
	if(it->flags & TRIG_CONFIG){
		do_pack(&s->io_bits,it);
		writel(s->io_bits,dev->iobase+Port_Pin_Directions(0));
	}else{
		if(it->flags&TRIG_WRITE){
			do_pack(&s->state,it);
			writel(s->state,dev->iobase+Port_IO(0));
		}else{
			unsigned int data;

			data=readl(dev->iobase+Port_IO(0));
			di_unpack(data,it);
		}
	}

	return it->n_chan;
}


int nidio_init(comedi_device *dev,comedi_devconfig *it)
{
	comedi_subdevice *s;
	int ret;
	
	if(strcmp("nidio",it->board_name))
		return 0;
	
	printk("comedi%d: nidio:",dev->minor);

	if((ret=alloc_private(dev,sizeof(nidio96_private)))<0)
		return ret;
	
	ret=nidio_find_device(dev,it);
	if(ret<0)goto out;

	dev->iobase=mite_setup(devpriv->mite);

	dev->driver_name="nidio";
	dev->board_name=nidio_boards[dev->board].name;
	dev->irq=mite_irq(devpriv->mite);
	printk(" %s",dev->board_name);

	dev->n_subdevices=1;
	if((ret=alloc_subdevices(dev))<0)
		return ret;

	if(!nidio_boards[dev->board].is_diodaq){
		subdev_8255_init(dev,dev->subdevices+0,nidio96_8255_cb,(void *)(dev->iobase+NIDIO_A));
		subdev_8255_init(dev,dev->subdevices+1,nidio96_8255_cb,(void *)(dev->iobase+NIDIO_B));
		subdev_8255_init(dev,dev->subdevices+2,nidio96_8255_cb,(void *)(dev->iobase+NIDIO_C));
		subdev_8255_init(dev,dev->subdevices+3,nidio96_8255_cb,(void *)(dev->iobase+NIDIO_D));
	}else{

		printk(" rev=%d",readb(dev->iobase+Chip_Version));
	
		s=dev->subdevices+0;

		s->type=COMEDI_SUBD_DIO;
		s->subdev_flags=SDF_READABLE|SDF_WRITEABLE|SDF_RT;
		s->n_chan=32;
		s->range_type=RANGE_digital;
		s->maxdata=1;
		s->trig[0]=nidio_dio;

		writel(0,dev->iobase+Port_IO(0));
		writel(0,dev->iobase+Port_Pin_Directions(0));
		writel(0,dev->iobase+Port_Pin_Mask(0));
	}

	dev->rem=nidio_rem;
	
	printk("\n");

	return 1;
out:
	nidio_rem(dev);

	return ret;
}

static int nidio_rem(comedi_device *dev)
{
	if(devpriv && devpriv->mite)
		mite_unsetup(devpriv->mite);

	return 0;
}


static int nidio_find_device(comedi_device *dev,comedi_devconfig *it)
{
	struct mite_struct *mite;
	int i;
	
	for(mite=mite_devices;mite;mite=mite->next){
		for(i=0;i<n_nidio_boards;i++){
			if(mite_device_id(mite)==nidio_boards[i].dev_id)
				dev->board=i;
				devpriv->mite=mite;

				return 0;
		}
	}
	printk("no device found\n");
	return -EIO;
}
